package ReadWrite;

public abstract class RWVT {
	protected int activeReaders_ = 0;																	// zero or more
	protected int activeWriters_ = 0;				 													// always zero or one
	protected int waitingReaders_ = 0;
	protected int waitingWriters_ = 0;
	protected abstract void read_();																	// defined by subclass
	protected abstract void write_(); 
	public void read() {										// unsynchronized
		beforeRead();													// obtain access
		read_();
		afterRead();													// release access
	}
	public void write() {
		beforeWrite();
		write_();
		afterWrite();
	}
	protected synchronized void beforeRead() {
		++waitingReaders_;										// available to subclasses
		while (!allowReader())
			try { wait(); }
			catch (InterruptedException ex) {}
		--waitingReaders_;
		++activeReaders_;
	}
	protected boolean allowReader() {																		// default policy
		return waitingWriters_ == 0 && activeWriters_ == 0;
	}
	protected synchronized void afterRead() { 
		--activeReaders_;
		notifyAll();
	}
	protected synchronized void beforeWrite() {
		++waitingWriters_;
		while (!allowWriter()) 
			try { wait(); }
			catch (InterruptedException ex) {}
		--waitingWriters_;
		++activeWriters_;
	}
	protected boolean allowWriter() {																		// default policy
		return activeReaders_ == 0 && activeWriters_ == 0;
	}
	protected synchronized void afterWrite() { 
		--activeWriters_;
		notifyAll();
	}
}
