/*
  File: Philosopher.java
*/

package dinersSol;

/** represents a passive Philosopher.  
 *  provide a thread for each philosopher to make it an active. 
 *  philosopher can:
 *  pickLeft()   pick their left fork
 *  pickRight()  pick their right fork 
 *			   they start eating when they have both forks. 
 *  setThinking() drop their forks and think
 */
public final class Philosopher 
{
	private int id_;
	private PhilTable view_;   
	private int state_;
	private Fork left_;
	private Fork right_;
	private boolean hasLeft_; 
	private boolean hasRight_;

	Philosopher( PhilTable view, int id, Fork l, Fork r ) 
	{
		view_ = view;
		id_ = id; 
		left_ = l; 
		right_ = r;
		hasLeft_ = false; 
		hasRight_ = false;
		init();
	}

	public int getId() 
	{
		return id_; 
	}
	
	public boolean isEven() {
		return (id_ % 2) == 0;
	}

	public int getState()
	{
		return state_;
	}

	private void updateState( int state )
	{
		state_ = state;
		view_.setPhilState( this );
	}

	public void init() 
	{
		if ( hasRight_ ) 
			{
				right_.init(); 
				hasRight_ = false; 
			} 
		if ( hasLeft_ ) 
			{
				left_.init(); 
				hasLeft_ = false; 
			}
		updateState( PhilTable.THINKING );
	}


	/** drops both Forks and start thinking
	 */
	public void setThinking()
	{
		if ( hasRight_ ) 
			{
				right_.put(); 
				hasRight_ = false; 
			} 
		if ( hasLeft_ ) 
			{
				left_.put(); 
				hasLeft_ = false; 
			}
		updateState( PhilTable.THINKING );
	}

	public void setHungry()
	{
		updateState( PhilTable.HUNGRY );
	}

	/** tries to pick the left fork:  
	 *  nothing happens when the philo. already has the left fork. 
	 *  goes into eating state when he also has the right fork. 
	 */
	public void pickLeft() throws InterruptedException 
	{
		if ( hasLeft_ ) 
			return;
		else
			{
				if ( hasRight_ ) 
					{
						left_.get(); 
						hasLeft_ = true; 
						updateState( PhilTable.EATING );
					}
				else 
					{
						left_.get(); 
						hasLeft_ = true; 
						updateState( PhilTable.GOTLEFT );
					}
			}
	}
 

  /** tries to pick the right fork:  
   *  nothing happens when the philo. already has the right fork. 
   *  goes into eating state when he also has the left fork. 
   */
	public void pickRight() throws InterruptedException 
	{
		if ( hasRight_ ) 
			return;
		else
			{
				if ( hasLeft_ ) 
					{
						right_.get(); 
						hasRight_ = true; 
						updateState( PhilTable.EATING );
					}
				else 
					{
						right_.get(); 
						hasRight_ = true; 
						updateState( PhilTable.GOTRIGHT );
					}
			}
	}

}
