/*
  File: RotatorPanel.java
*/

package doublerotatorSol;

import java.awt.*;
import java.awt.event.*;
import support.*;

public class RotatorPanel extends Panel implements Runnable
{
	ArcPanel arcPanel_;
	Button run_;
	Button pause_;
	boolean isRunning_ = false;

	static final Color RUNCOLOR = Color.green;
	static final Color PAUSECOLOR = Color.orange;

	public RotatorPanel( String title )
	{
		this( new BorderLayout(), title );
	}

	public RotatorPanel( LayoutManager layout, String title )
	{
		super( layout );

		Panel p = new Panel();
		run_ = new Button( "Run" );
		run_.addActionListener( new RunListener( this ) );
		pause_ = new Button( "Pause" );
		pause_.addActionListener( new PauseListener( this ) );
		p.add( run_ );
		p.add( pause_ );

		arcPanel_ = new ArcPanel( title, PAUSECOLOR );
		add( arcPanel_, BorderLayout.NORTH );
		add( p, BorderLayout.SOUTH );
	}
	
	public void run() {
	while(isRunning()) {
			arcPanel_.rotate();
			try {
					Thread.sleep(100);
				} catch(InterruptedException e) { }
				}
	}
	
	public synchronized boolean isRunning() {
			while (!isRunning_) {
					try { wait(); }
					catch (InterruptedException e) { return false; }
			}
			return true;
		}

	public void setZero()
	{
		// arcPanel_.setZero();
		setInActive();
	}

	public synchronized void setActive()
	{
		isRunning_ = true;
		run_.setEnabled( false );
		pause_.setEnabled( true );
		// arcPanel_.setArcColor( RUNCOLOR );
		// this.setBackground( RUNCOLOR );
		arcPanel_.setBackground( RUNCOLOR );
		arcPanel_.repaint();
		notifyAll();
	}

	public synchronized void setInActive()
	{
		isRunning_ = false;
		run_.setEnabled( true );
		pause_.setEnabled( false );
		// arcPanel_.setArcColor( PAUSECOLOR );
		// this.setBackground( PAUSECOLOR );
		arcPanel_.setBackground( PAUSECOLOR );
		arcPanel_.repaint();
		// notifyAll();
	}

	public void start() 
	{
		setZero();
		new Thread(this).start();
	}

	public void stop() 
	{
	}

	class RunListener implements ActionListener
	{
		RotatorPanel owner_;

		public RunListener( RotatorPanel owner )
		{
			owner_ = owner;
		}

		public void actionPerformed( ActionEvent evt )
		{
			owner_.setActive();
		}

	}

	class PauseListener implements ActionListener
	{
		RotatorPanel owner_;

		public PauseListener( RotatorPanel owner )
		{
			owner_ = owner;
		}

		public void actionPerformed( ActionEvent evt )
		{
			owner_.setInActive();
		}

	}

}
