/*
@author  j.n.magee 25/04/98
*/
package golfSol;

import java.awt.*;
import java.applet.*;

public class GolfClub extends Applet 
{
	SlotCanvas waiting_;
	SlotCanvas playing_;
	DisplayAllocator allocator_;

	final static int PLAYTIME = 10000;
	final static int CAPACITY = 5;
	final static int MAX_PLAYING = CAPACITY; 
	final static int MAX_WAITING = CAPACITY; 

	/** a Player can take at most NO_BALL
	 *  Warning: NO_BALL should not be bigger than CAPACITY 
	 */
	final static int NO_BALL = 5; 

	/** factory method to create Allocator for this GolfClub. 
	 *  overwrite it and create more sophisticated Allocators.  
	 */
	protected IAllocator buildAllocator() {
		return new SimpleAllocator( CAPACITY ); 
	}

	public void init() 
	{
		setLayout( new BorderLayout() );

		allocator_ = new DisplayAllocator(buildAllocator());

		Panel scene = new Panel(new BorderLayout());
		scene.setBackground( Color.orange );
		
		waiting_ = new SlotCanvas( "waiting bar", Color.yellow, MAX_WAITING );
		playing_ = new SlotCanvas( "playing..", Color.green, MAX_PLAYING );
		Panel slots = new Panel(); 
		
		slots.add( waiting_ ); 
		slots.add( playing_ ); 
		scene.add( allocator_, BorderLayout.NORTH );
		scene.add( slots, BorderLayout.CENTER );
		add( scene, BorderLayout.NORTH );


		add( new PlayerArrival( this, NO_BALL ), BorderLayout.CENTER );
	}

	/** a player with a name wants to get n balls
	 *  return false to signal that the player cannot be served, 
	 *  nor put in the waiting hall. 
	 */
	boolean getGolfBalls( int n,  String name ) throws InterruptedException 
	{
		String s = name + n;
		boolean waitingInBar = waiting_.enter( s );
		if ( !waitingInBar ) 
		{
			showStatus( "No space in the bar for player " + s ); 
			return false; 
		} else {
			showStatus(""); 
		}
		allocator_.get( n );
		waiting_.leave( s );
		playing_.enter( s );
		return true; 
	}


	/** returns the time a player may play. 
	 *  Could be a random number in a range 
	 */
	public int getPlayTime() 
	{
		return PLAYTIME;
	}

	/** a player with name releases n balls
	 */
	void relGolfBalls( int n,  String name )
			  throws InterruptedException 
	{
		String s = name + n;
		allocator_.put( n );
		playing_.leave( s );
	}

}

