/*
  File: PhilTable.java
*/

package diners;

import java.awt.*;
import java.applet.*; 
import java.net.*; 

/** View for Philosopers and Forks.  
 *  Philosophers and Forks inform view about changes
 */

public class PhilTable extends Canvas
{

	static final int NUMPHILS = 5;

	static final int THINKING = 0;
	static final int HUNGRY = 1;
	static final int GOTRIGHT = 2;
	static final int EATING = 3;
	static final int GOTLEFT = 4;

	private Image[] imgs_ = new Image[5];
	
	private double [] philX_  = new double[NUMPHILS];
	private double [] philY_  = new double[NUMPHILS];
	
	private int [] philstate_ = new int [NUMPHILS];
	
	private double [] forkX_  = new double[NUMPHILS];
	private double [] forkY_  = new double[NUMPHILS];

	private boolean[] forkstate_ = new boolean[NUMPHILS];

	public PhilTable( Applet applet ) 
	{
		MediaTracker mt;
		URL documentBase = applet.getDocumentBase(); 

		setSize( 300, 320 ); // set initial size

		mt = new MediaTracker( this );

		imgs_[THINKING] = 
			applet.getImage( documentBase, "Images/thinking.gif" );
		mt.addImage( imgs_[THINKING], THINKING );
		imgs_[HUNGRY] = 
			applet.getImage( documentBase, "Images/hungry.gif" );
		mt.addImage( imgs_[HUNGRY], HUNGRY );
		imgs_[GOTRIGHT] = 
			applet.getImage( documentBase, "Images/gotright.gif" );
		mt.addImage( imgs_[GOTRIGHT], GOTRIGHT );
		imgs_[EATING] = 
			applet.getImage( documentBase, "Images/eating.gif" );
		mt.addImage( imgs_[EATING], EATING );
		imgs_[GOTLEFT] = 
			applet.getImage( documentBase, "Images/gotleft.gif" );
		mt.addImage( imgs_[GOTLEFT], GOTLEFT );
		
		try 
			{
				mt.waitForAll();
			} 
		catch ( InterruptedException e) 
			{
				System.out.println( "Couldn't load one of the images" );
			}
		initPlacing();
	}

	public void start()
	{
		setBackground( Color.lightGray );
		repaint();
	}

	public void setFrozen()
	{
		setBackground( Color.red );
		repaint();
	}

	public synchronized void setForkState( Fork f )
	{
		forkstate_[f.getId()] = f.getState();
		// repaint not needed (protocol)
	}
		
	public synchronized void setPhilState( Philosopher p )
	{
		philstate_[p.getId()] = p.getState();
		repaint();
	}

	boolean canupdate_ = true;


	public void repaint()
	{
		if ( canupdate_ )
			{
				super.repaint();
			}
	}

	public void enableUpdate()
	{
		canupdate_ = true;
		repaint();
	}
	
	public void disableUpdate()
	{
		canupdate_ = false;
	}

	/** place philosophers around the table
	 */
	private void initPlacing() 
	{
		double x, y;
		double radius = 105.0;
		double centerAdj = 100.0;
		double radians;

		for ( int i = 0; i < NUMPHILS; i++ ) 
			{
				radians = i * (2.0 * Math.PI / (double)NUMPHILS);
				philX_[i] = Math.sin(radians) * radius + centerAdj;
				philY_[i] = Math.cos(radians) * radius + centerAdj;
			}

		radius = 35;
		centerAdj = 145;
		
		for ( int i = 0; i < NUMPHILS; i++ )
			{
				radians = i * (2 * Math.PI / (double)NUMPHILS) + Math.PI / 5;
				forkX_[i] = Math.sin(radians) * radius + centerAdj;
				forkY_[i] = Math.cos(radians) * radius + centerAdj;
			}
	}

	private Image offscreen_;
	private Dimension offscreensize_;
	private Graphics offgraphics_;

	private void prepare() 
	{
		Dimension d = getSize();
		if ( (offscreen_ == null) || 
			 (d.width != offscreensize_.width) || 
			 (d.height != offscreensize_.height) ) 
			{
				offscreen_ = createImage( d.width, d.height );
				offscreensize_ = d;
				offgraphics_ = offscreen_.getGraphics();
				offgraphics_.setFont(new Font("Helvetica",Font.BOLD,18));
			}
	}

	public void paint( Graphics g ) 
	{
		prepare();
		update( g );
	}
		
	private void drawTable()
	{
		offgraphics_.setColor( Color.white );
		offgraphics_.fillOval( 105, 105, 90, 90 );
		offgraphics_.setColor( Color.black );
		for( int i = 0; i < NUMPHILS; i++ ) 
			{
				if ( !forkstate_[i] ) 
					offgraphics_.fillOval( (int)forkX_[i],
										   (int)forkY_[i],
										   10,
										   10 );
			}
	}

	private void drawPhils()
	{
		offgraphics_.setColor( getBackground() );
		for ( int i = 0; i < NUMPHILS; i++ ) 
			{
				offgraphics_.fillRect( (int)philX_[i], 
									   (int)philY_[i],
									   imgs_[0].getWidth( this ),
									   imgs_[0].getHeight( this ) );
				offgraphics_.drawImage( imgs_[philstate_[i]], 
										(int)philX_[i], 
										(int)philY_[i], 
										this );
			}
	}

	public void update( Graphics g ) 
	{
		if ( offscreen_ == null) 
			return; // something went wrong

		boolean d = deadlocked();

		if ( d )
			setBackground( Color.red );
		offgraphics_.setColor( getBackground() );
		offgraphics_.fillRect( 0, 0, getSize().width, getSize().height );
		
		drawTable();
		drawPhils();

		// check if deadlock has been reached
		if ( d ) 
			{
				offgraphics_.setColor( Color.black );
				offgraphics_.drawString( "DEADLOCKED" , 90, 130 );
			}
 
		g.drawImage( offscreen_, 0, 0, null );
	}
 
	private boolean deadlocked()
	{
		int i = 0;

		while ( i < NUMPHILS && 
				(philstate_[i] == GOTRIGHT || philstate_[i] == GOTLEFT) ) 
			i++;
		return i == NUMPHILS;
	}
		
}
