/*
  File: RunnablePhilosopher.java
*/

package diners; 

/**
 *  a runnable Philosopher
 * 
 *  Strategy: a philosoper thinks, picks right and left fork, eats 
 *  forever
 *
 *  This class runs each philosopher in its own thread and provides
 *  the eating, thinking strategy.
 */

public class RunnablePhilosopher extends Thread 
{
	protected final Philosopher phil_; 

	protected final Diners controller_; 
	
	boolean terminated_ = false; 
	
	public RunnablePhilosopher( Philosopher p, Diners controller ) 
	{
		super(); 
		phil_ = p; 
		controller_ = controller;
		phil_.init();
	}

	public void terminate() 
	{
		interrupt();
	}

	public boolean isTerminated()
	{
		return terminated_;
	}

	public void start()
	{
		super.start();
	}

	public void interrupt()
	{
		terminated_ = true;
		super.interrupt();
	}

	private void pause() throws InterruptedException
	{
		sleep( 500 );
	}

	private void think() throws InterruptedException
	{
		sleep( controller_.getThinkTime() );
	}

	private void eat() throws InterruptedException
	{
		sleep( controller_.getEatTime() );
	}

	public void run() 
	{
		try 
			{
				while ( !isTerminated() ) 
					{
						phil_.setThinking();
						think();
						phil_.setHungry();
						phil_.pickRight(); 
						pause();
						phil_.pickLeft(); 
						eat();
					}
			} 
		catch (InterruptedException e) 
			{
				return;
			}
	}

}


